@@ -3,7 +3,6 @@ package main
 import (
 	"context"
 	"crypto/tls"
-	"flag"
 	"fmt"
 	"net/http"
 	"os"
@@ -13,16 +12,18 @@ import (
 	configclientset "github.com/openshift/client-go/config/clientset/versioned"
 	configv1client "github.com/openshift/client-go/config/clientset/versioned/typed/config/v1"
 	"github.com/prometheus/client_golang/prometheus/promhttp"
-	log "github.com/sirupsen/logrus"
+	"github.com/sirupsen/logrus"
+	"github.com/spf13/pflag"
 	v1 "k8s.io/api/core/v1"
-	"k8s.io/client-go/tools/clientcmd"
 
-	"github.com/operator-framework/operator-lifecycle-manager/pkg/api/client"
+	"github.com/operator-framework/operator-lifecycle-manager/pkg/api/client/clientset/versioned"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/controller/operators/olm"
+	"github.com/operator-framework/operator-lifecycle-manager/pkg/feature"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/lib/filemonitor"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/lib/operatorclient"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/lib/operatorstatus"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/lib/profile"
+	"github.com/operator-framework/operator-lifecycle-manager/pkg/lib/queueinformer"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/lib/signals"
 	"github.com/operator-framework/operator-lifecycle-manager/pkg/metrics"
 	olmversion "github.com/operator-framework/operator-lifecycle-manager/pkg/version"
@@ -36,43 +37,43 @@ const (
 
 // config flags defined globally so that they appear on the test binary as well
 var (
-	kubeConfigPath = flag.String(
-		"kubeconfig", "", "absolute path to the kubeconfig file")
-
-	wakeupInterval = flag.Duration(
+	wakeupInterval = pflag.Duration(
 		"interval", defaultWakeupInterval, "wake up interval")
 
-	watchedNamespaces = flag.String(
+	watchedNamespaces = pflag.String(
 		"watchedNamespaces", "", "comma separated list of namespaces for olm operator to watch. "+
 			"If not set, or set to the empty string (e.g. `-watchedNamespaces=\"\"`), "+
 			"olm operator will watch all namespaces in the cluster.")
 
-	writeStatusName = flag.String(
+	writeStatusName = pflag.String(
 		"writeStatusName", defaultOperatorName, "ClusterOperator name in which to write status, set to \"\" to disable.")
 
-	writePackageServerStatusName = flag.String(
+	writePackageServerStatusName = pflag.String(
 		"writePackageServerStatusName", defaultPackageServerStatusName, "ClusterOperator name in which to write status for package API server, set to \"\" to disable.")
 
-	debug = flag.Bool(
+	debug = pflag.Bool(
 		"debug", false, "use debug log level")
 
-	version = flag.Bool("version", false, "displays olm version")
+	version = pflag.Bool("version", false, "displays olm version")
 
-	tlsKeyPath = flag.String(
+	tlsKeyPath = pflag.String(
 		"tls-key", "", "Path to use for private key (requires tls-cert)")
 
-	tlsCertPath = flag.String(
+	tlsCertPath = pflag.String(
 		"tls-cert", "", "Path to use for certificate key (requires tls-key)")
 
-	profiling = flag.Bool(
+	profiling = pflag.Bool(
 		"profiling", false, "serve profiling data (on port 8080)")
 
-	namespace = flag.String(
+	namespace = pflag.String(
 		"namespace", "", "namespace where cleanup runs")
 )
 
 func init() {
 	metrics.RegisterOLM()
+
+	// Add feature gates before parsing
+	feature.AddFlag(pflag.CommandLine)
 }
 
 // main function - entrypoint to OLM operator
@@ -80,9 +81,9 @@ func main() {
 	// Get exit signal context
 	ctx, cancel := context.WithCancel(signals.Context())
 	defer cancel()
+	pflag.Parse()
 
 	// Parse the command-line flags.
-	flag.Parse()
 
 	// Check if version flag was set
 	if *version {
@@ -103,9 +104,9 @@ func main() {
 	}
 
 	// Set log level to debug if `debug` flag set
-	logger := log.New()
+	logger := logrus.New()
 	if *debug {
-		logger.SetLevel(log.DebugLevel)
+		logger.SetLevel(logrus.DebugLevel)
 	}
 	logger.Infof("log level %s", logger.Level)
 
@@ -168,24 +169,27 @@ func main() {
 		}()
 	}
 
-	// create a config client for operator status
-	config, err := clientcmd.BuildConfigFromFlags("", *kubeConfigPath)
+	mgr, err := Manager(ctx)
 	if err != nil {
-		log.Fatalf("error configuring client: %s", err.Error())
+		logger.WithError(err).Fatalf("error configuring controller manager")
 	}
+	config := mgr.GetConfig()
+
 	versionedConfigClient, err := configclientset.NewForConfig(config)
 	if err != nil {
-		err = fmt.Errorf("error configuring OpenShift Proxy client: %v", err)
-		return
+		logger.WithError(err).Fatal("error configuring openshift proxy client")
 	}
 	configClient, err := configv1client.NewForConfig(config)
 	if err != nil {
-		log.Fatalf("error configuring client: %s", err.Error())
+		logger.WithError(err).Fatal("error configuring config client")
 	}
-	opClient := operatorclient.NewClientFromConfig(*kubeConfigPath, logger)
-	crClient, err := client.NewClient(*kubeConfigPath)
+	opClient, err := operatorclient.NewClientFromRestConfig(config)
 	if err != nil {
-		log.Fatalf("error configuring client: %s", err.Error())
+		logger.WithError(err).Fatal("error configuring operator client")
+	}
+	crClient, err := versioned.NewForConfig(config)
+	if err != nil {
+		logger.WithError(err).Fatal("error configuring custom resource client")
 	}
 
 	cleanup(logger, opClient, crClient)
@@ -195,14 +199,14 @@ func main() {
 		ctx,
 		olm.WithLogger(logger),
 		olm.WithWatchedNamespaces(namespaces...),
-		olm.WithResyncPeriod(*wakeupInterval),
+		olm.WithResyncPeriod(queueinformer.ResyncWithJitter(*wakeupInterval, 0.2)),
 		olm.WithExternalClient(crClient),
 		olm.WithOperatorClient(opClient),
 		olm.WithRestConfig(config),
 		olm.WithConfigClient(versionedConfigClient),
 	)
 	if err != nil {
-		log.WithError(err).Fatalf("error configuring operator")
+		logger.WithError(err).Fatalf("error configuring operator")
 		return
 	}
 
@@ -226,5 +230,10 @@ func main() {
 		go monitor.Run(op.Done())
 	}
 
+	// Start the controller manager
+	if err := mgr.Start(ctx.Done()); err != nil {
+		logger.WithError(err).Fatal("controller manager stopped")
+	}
+
 	<-op.Done()
 }
